
import java.io.*;
import java.util.*;

// Methods for generating html-files
class ttchess_html extends ttchess_common {

	static String extension = ".html";
	static String indexfilename = "board_index";
	static String historyprefix = "boardhistory_";
	static String snapshotprefix = "boardsnapshot_";

	// Generates the index-file of all boards and the time of their latest moves
	static void print_index(ttchess_database db) {

		// Generate the content first, then quickly open the file and write everything.
		StringBuffer s = new StringBuffer(2000);
		s.append("<html>\n<head>\n  <meta http-equiv=refresh content=3>\n</head>\n");  // 1 second refreshrate
		s.append("<body><h1>Chessboards</h1>\n<table border=1 cellspacing=0 cellpadding=10>\n");
		s.append("<tr><th>Board id<th>Status<th>Time of last move<th>Snapshot<th>History\n");

		for (Enumeration e = db.get_boards() ; e.hasMoreElements() ;) {
			ttchess_chessboard b = (ttchess_chessboard)(e.nextElement());

			s.append("<tr>\n<td valign=top>");
			s.append(Integer.toString(b.get_address()));
			s.append("\n<td>");
			if (b.get_alive()==true) s.append("Online"); else s.append("Offline");
			s.append("<td>");
			s.append(b.get_snapshot().get_time().toString());
			s.append("\n<td><a href=\"");
			s.append(snapshotprefix);
			s.append(Integer.toString(b.get_address()));
			s.append(extension);
			s.append("\">Link</a><td><a href=\"");
			s.append(historyprefix);
			s.append(Integer.toString(b.get_address()));
			s.append(extension);
			s.append("\">Link</a>\n");

		}
		s.append("</table>\n</body>\n</html>\n");

		if (writefile(indexfilename+extension, s.toString()) == false) {
			System.err.println("Error: Failed to write indexfile.");
			return;
		}
	}

	// Generates the history-page for one board
	static void print_history(ttchess_history h) {
		// Generate the content first, then quickly open the file and write everything.
		StringBuffer s = new StringBuffer(2000);
		s.append("<html>\n<head></head>\n");
		s.append("<body><h1>History for Chessboard: ");
		s.append(Integer.toString(h.board.get_address()));
		s.append("</h1>\n");

		int i = 0;
		for (Enumeration e = h.history.elements() ; e.hasMoreElements() ;) {
			ttchess_snapshot ss = (ttchess_snapshot)(e.nextElement());
			i=i+1;
			s.append("<p><hr size=1><p><h3>Move: ");
			s.append(Integer.toString(i));
			s.append(" &nbsp; &nbsp;");
			s.append(ss.get_time().toString());
			s.append(boardview(ss));
			s.append("\n");
		}
		s.append("</body>\n</html>\n");

		if (writefile(historyprefix + Integer.toString(h.board.get_address()) + extension, s.toString()) == false) {
			System.err.println("Error: Failed to write historyfile.");
			return;
		}
	}

	// Generates a string with a html table of the board
	static String boardview(ttchess_snapshot b) {
		StringBuffer s = new StringBuffer(1000);
		s.append("<table border=0 cellpadding=0 cellspacing=0>\n");
		boolean filled = true;
		for (int i=0; i<8; i=i+1) {
			s.append("<tr>");
			for (int j=0; j<8; j=j+1) {
				s.append("<td><img src=\"images/");
				int p = b.pieces[(i*8)+j];
				if (p==EMPTY  ) s.append("empty");
				else if (p==WPAWN  ) s.append("w_pawn");
				else if (p==WROOK  ) s.append("w_rook");
				else if (p==WKNIGHT) s.append("w_knight");
				else if (p==WBISHOP) s.append("w_bishop");
				else if (p==WKING  ) s.append("w_king");
				else if (p==WQUEEN ) s.append("w_queen");
				else if (p==BPAWN  ) s.append("b_pawn");
				else if (p==BROOK  ) s.append("b_rook");
				else if (p==BKNIGHT) s.append("b_knight");
				else if (p==BBISHOP) s.append("b_bishop");
				else if (p==BKING  ) s.append("b_king");
				else if (p==BQUEEN ) s.append("b_queen");
				if (filled) s.append("1"); else s.append("2");
				filled = !filled;
				s.append(".gif\">");
			}
			filled = !filled;
			s.append("</tr>\n");
		}
		s.append("\n</table>");
		return s.toString();
	}


	// Generates the snapshot page for one board
	static void print_snapshot(ttchess_snapshot ss) {
		// Generate the content first, then quickly open the file and write everything.
		StringBuffer s = new StringBuffer(2000);
		s.append("<html>\n<head>\n  <meta http-equiv=refresh content=4>\n</head>\n");
		s.append("<body><h1>Snapshot of Chessboard: ");
		s.append(Integer.toString(ss.board.get_address()));
		s.append("</h1>\n<h3>");
		s.append(ss.get_time().toString());
		s.append("</h3>\n");
		s.append(boardview(ss));
		s.append("\n</body>\n</html>\n");

		if (writefile(snapshotprefix + Integer.toString(ss.board.get_address()) + extension, s.toString()) == false) {
			System.err.println("Error: Failed to write snapshotfile.");
			return;
		}
	}

	// Tries several times to open a file, in case it is locked by another process (the web server)
	static boolean writefile(String filename, String content) {
		FileWriter f = null;
		for (int i=0; i<10; i=i+1) {  // number of retries
			try {
				f = new FileWriter(filename);
				break;
			}
			catch (Throwable t) {
				System.err.println("Error opening "+filename+" for writing. Retrying...");
			}
		}
		if (f==null) {
			System.out.println("Failed to open file "+filename);
			return false;
		}
		try {
			f.write(content);
			f.close();
		}
		catch (Exception e) {
			System.out.println("Error writing file "+filename+". ("+e.toString()+")");
			return false;
		}
		return true;		
	}
}
